<?php

namespace Drupal\media_folder_management\Controller;

use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Url;
use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Cache\CacheableMetadata;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\media_folder_management\MediaFolderFileUiActions;
use Drupal\media_folder_management\MediaFolderFileUiBuilder;

/**
 * Class File media folder file Controller.
 */
class MediaFolderFileController extends ControllerBase {

  /**
   * The folders ui service.
   *
   * @var \Drupal\media_folder_management\MediaFolderFileUiBuilder
   */
  protected $foldersUi;

  /**
   * The actions ui service.
   *
   * @var \Drupal\media_folder_management\MediaFolderFileUiActions
   */
  protected $actionsUi;

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('media_folder_management.ui_builder'),
      $container->get('media_folder_management.ui_actions'),
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(
    MediaFolderFileUiBuilder $media_folder_management_ui,
    MediaFolderFileUiActions $media_folder_management_actions,
    RequestStack $request_stack,
  ) {
    $this->foldersUi = $media_folder_management_ui;
    $this->actionsUi = $media_folder_management_actions;
    $this->requestStack = $request_stack;
  }

  /**
   * Get title.
   */
  public function getTitle(mixed $folder = NULL) {
    return $this->foldersUi->getTitle($folder);
  }

  /**
   * Display media folder.
   */
  public function fileExplorer($folder = NULL) {
    if ($folder && ($folder->bundle() !== 'media_folder')) {
      throw new NotFoundHttpException();
    }

    return $this->foldersUi->buildAdminUi($folder);
  }

  /**
   * Display ajax Media folder.
   */
  public function fileExplorerAjax($folder = NULL) {
    if ($folder && ($folder->bundle() !== 'media_folder')) {
      throw new NotFoundHttpException();
    }

    $build = [];
    $output = $this->foldersUi->buildAjaxResponse($folder, $build);
    $response = new CacheableJsonResponse($output, 200);
    $response->addCacheableDependency(CacheableMetadata::createFromRenderArray($build));
    if ($folder) {
      $response->addCacheableDependency($folder);
    }

    return $response;
  }

  /**
   * Display ajax Media folder.
   */
  public function fileExplorerWidgetAjax($folder, $entity_type_id, $bundle, $widget_id) {
    if ($folder->bundle() !== 'media_folder') {
      throw new NotFoundHttpException();
    }

    $build = [];
    $request = $this->requestStack->getCurrentRequest()->request->all();
    $output = $this->foldersUi->buildWidgetAjaxResponse($folder, $request, $entity_type_id, $bundle, $widget_id, $build);
    $response = new CacheableJsonResponse($output, 200);
    $response->addCacheableDependency(CacheableMetadata::createFromRenderArray($build));
    if ($folder) {
      $response->addCacheableDependency($folder);
    }

    return $response;
  }

  /**
   * Media folder Perms.
   */
  public function fileExplorerPerms($folder) {
    return $this->foldersUi->fileExplorerPerms($folder);
  }

  /**
   * Media folder share link.
   */
  public function share($folder) {
    $url = Url::fromRoute('entity.taxonomy_term.canonical', ['taxonomy_term' => $folder->id()], ['absolute' => TRUE])->toString();
    return [
      '#theme' => 'media_folder_share',
      '#link' => $url,
      '#cache' => [
        'tags' => [
          'taxonomy_term:' . $folder->id(),
        ],
      ],
      '#attached' => [
        'library' => ['media_folder_management/folders'],
      ],
    ];
  }

  /**
   * Ajax upload files to folder.
   */
  public function ajaxUploadFile($folder) {
    $request = $this->requestStack->getCurrentRequest()->request->all();
    $response = $this->actionsUi->ajaxUploadFile($folder, $request);

    return new JsonResponse($response);
  }

  /**
   * Ajax upload files to folder.
   */
  public function ajaxMoveInto($folder) {
    $request = $this->requestStack->getCurrentRequest()->request->all();
    $response = $this->actionsUi->ajaxMoveInto($folder, $request);

    return new JsonResponse($response);
  }

  /**
   * Ajax update widget.
   */
  public function fileExplorerUpdateWidgetAjax($folder, $media_folder_file, $widget_id) {
    $response = [
      'widget_id' => $widget_id,
      'fid' => $media_folder_file->id(),
    ];

    return new JsonResponse($response);
  }

  /**
   * Displays media details via AJAX.
   */
  public function ajaxMediaDetail($media_folder_file) : CacheableJsonResponse {
    $build = [
      '#cache' => [
        'tags' => ['media:' . $media_folder_file->id()],
      ],
    ];
    $output = $this->foldersUi->ajaxMediaDetail($media_folder_file);

    $response = new CacheableJsonResponse($output, 200);
    $response->addCacheableDependency(CacheableMetadata::createFromRenderArray($build));
    if ($media_folder_file) {
      $response->addCacheableDependency($media_folder_file);
    }

    return $response;
  }

}
