<?php

namespace Drupal\media_folder_management\Entity\Access;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;
use Drupal\taxonomy\Entity\Term;

/**
 * Class File media folder file Controller.
 */
class MediaFolderFileAccessCheck extends ControllerBase {

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityManager;

  /**
   * Current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The term storage handler.
   *
   * @var \Drupal\taxonomy\TermStorageInterface
   */
  protected $storageController;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(
    EntityTypeManagerInterface $entity_manager,
    AccountInterface $current_user,
  ) {
    $this->entityManager = $entity_manager;
    $this->currentUser = $entity_manager->getStorage('user')->load($current_user->id());
    $this->storageController = $entity_manager->getStorage('taxonomy_term');
  }

  /**
   * Account has admin permission.
   */
  public static function hasAdminPermission($account) {
    if ($account->hasPermission('bypass media folder files permissions') || $account->hasPermission('administer folders') || $account->hasPermission('administer media folder files')) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Account has admin folder permission.
   */
  public static function hasAdminFolderPermission($account) {
    if ($account->hasPermission('administer folders') || $account->hasPermission('administer media folder files')) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Account has admin folder permission.
   */
  public static function hasCreateRootFolderPermission($account = FALSE) {
    if (!$account) {
      $account = \Drupal::currentUser();
    }

    if ($account->hasPermission('administer folders') || $account->hasPermission('administer media folder files')) {
      return AccessResult::allowed()->cachePerPermissions();
    }

    return AccessResult::forbidden()->cachePerPermissions();
  }

  /**
   * Account has create folder permission.
   */
  public static function hasCreateFolderPermission($folder, $account = FALSE, $bool = FALSE) {
    if (!$account) {
      $account = \Drupal::currentUser();
    }

    if ($folder) {
      if (is_numeric($folder)) {
        $folder = Term::load($folder);
      }

      if ($account->hasPermission('administer own folders') || self::hasFolderPermission('create', $folder, $account)) {
        return ($bool) ? TRUE : AccessResult::allowed()->cachePerPermissions();
      }
      return ($bool) ? FALSE : AccessResult::forbidden()->cachePerPermissions();
    }

    if ($account->hasPermission('administer own folders') || self::hasFolderPermission('create', $folder, $account)) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Account has view folder permission.
   */
  public static function hasViewFolderPermission($folder, $account = FALSE, $bool = FALSE) {
    if (!$account) {
      $account = \Drupal::currentUser();
    }

    if ($folder) {
      if (is_numeric($folder)) {
        $folder = Term::load($folder);
      }

      if ($account->hasPermission('administer own folders') || self::hasFolderPermission('view', $folder, $account)) {
        return ($bool) ? TRUE : AccessResult::allowed()->cachePerPermissions();
      }

      return ($bool) ? FALSE : AccessResult::forbidden()->cachePerPermissions();
    }

    return ($bool) ? FALSE : AccessResult::forbidden()->cachePerPermissions();
  }

  /**
   * Account has edit folder permission.
   */
  public static function hasEditFolderPermission($folder, $account = FALSE, $bool = FALSE) {
    if (!$account) {
      $account = \Drupal::currentUser();
    }

    if ($folder) {
      if (is_numeric($folder)) {
        $folder = Term::load($folder);
      }

      if ($account->hasPermission('administer own folders') || self::hasFolderPermission('edit_any', $folder, $account)) {
        return ($bool) ? TRUE : AccessResult::allowed()->cachePerPermissions();
      }
      elseif (!empty($folder->field_media_folder_uid->target_id) && $folder->field_media_folder_uid->target_id == $account->id() && self::hasFolderPermission('edit_own', $folder, $account)) {
        return ($bool) ? TRUE : AccessResult::allowed()->cachePerPermissions();
      }

      return ($bool) ? FALSE : AccessResult::forbidden()->cachePerPermissions();
    }

    return ($bool) ? FALSE : AccessResult::forbidden()->cachePerPermissions();
  }

  /**
   * Account has delete folder permission.
   */
  public static function hasDeleteFolderPermission($folder, $account = FALSE, $bool = FALSE) {
    if (!$account) {
      $account = \Drupal::currentUser();
    }

    if ($folder) {
      if (is_numeric($folder)) {
        $folder = Term::load($folder);
      }

      if ($account->hasPermission('administer own folders') || self::hasFolderPermission('delete', $folder, $account)) {
        return ($bool) ? TRUE : AccessResult::allowed()->cachePerPermissions();
      }
      elseif (!empty($folder->field_media_folder_uid->target_id) && $folder->field_media_folder_uid->target_id == $account->id() && self::hasFolderPermission('delete_own', $folder, $account)) {
        return ($bool) ? TRUE : AccessResult::allowed()->cachePerPermissions();
      }
      return ($bool) ? FALSE : AccessResult::forbidden()->cachePerPermissions();
    }

    return ($bool) ? FALSE : AccessResult::forbidden()->cachePerPermissions();
  }

  /**
   * Get parent permission.
   */
  public static function getParentPermissionTarget($folder, $permission) {
    if (!empty($folder->parent->entity) && $folder->field_perms_inherit->value) {
      if (!empty($folder->parent->entity->parent->entity) && $folder->parent->entity->field_perms_inherit->value) {
        $values = self::getParentPermissionTarget($folder->parent->entity, $permission);
      }
      else {
        $values = $folder->parent->entity->get($permission)->getValue();
      }
    }
    else {
      $values = $folder->get($permission)->getValue();
    }

    return $values;
  }

  /**
   * Account has folder permission.
   */
  public static function hasFolderPermission($permission, $folder, $account) {
    if (self::hasAdminPermission($account)) {
      return TRUE;
    }

    $values = self::getParentPermissionTarget($folder, 'field_perms_' . $permission);

    $roles = $account->getRoles();
    foreach ($values as $value) {
      if (in_array($value['target_id'], $roles)) {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Account has add file permission.
   */
  public static function accessAddFileToFolder($account) {
    $route_name = \Drupal::routeMatch()->getRouteName();
    if ($route_name == 'entity.media_folder_file.add_form') {
      $folder = \Drupal::routeMatch()->getParameter('folder');
      $storageController = \Drupal::service('entity_type.manager')->getStorage("taxonomy_term");

      if (self::hasAdminPermission($account)) {
        return AccessResult::allowed()->cachePerPermissions();
      }

      if ($account->hasPermission('access media folder file explorer')) {
        $ancestors = $storageController->loadAllParents($folder);
        foreach ($ancestors as $ancestor) {
          if (!self::hasFolderPermission('view', $ancestor, $account)) {
            return AccessResult::forbidden()->cachePerPermissions();
          }
        }

        $folder_term = $storageController->load($folder);
        if (self::hasFolderPermission('create', $folder_term, $account)) {
          return AccessResult::allowed()->cachePerPermissions();
        }
      }
    }

    return AccessResult::forbidden()->cachePerPermissions();
  }

  /**
   * Account has recursive permission.
   */
  public static function accessFolderRecursive($folder, $account, $storageController) {
    $ancestors = $storageController->loadAllParents($folder);
    foreach ($ancestors as $ancestor) {
      if (!self::hasFolderPermission('view', $ancestor, $account)) {
        return FALSE;
      }
    }

    return TRUE;
  }

  /**
   * Account has access permission.
   */
  public function accessFolder($folder) {
    if ($this->hasAdminPermission($this->currentUser)) {
      return AccessResult::allowed()->cachePerPermissions();
    }

    if ($this->currentUser->hasPermission('access media folder file explorer')) {
      if (is_numeric($folder)) {
        $folder = $this->storageController->load($folder);
      }
      if (!$this->accessFolderRecursive($folder->id(), $this->currentUser, $this->storageController)) {
        return AccessResult::forbidden()->cachePerPermissions();
      }
      return AccessResult::allowed()->cachePerPermissions();
    }

    return AccessResult::forbidden()->cachePerPermissions();
  }

  /**
   * Account has access folder permission.
   */
  public function accessFolderPerms($folder) {
    if ($this->hasAdminPermission($this->currentUser)) {
      return AccessResult::allowed()->cachePerPermissions();
    }

    return AccessResult::forbidden()->cachePerPermissions();
  }

  /**
   * Account has view file permission.
   */
  public static function hasViewFileAccess($file, $account, $storageController = NULL) {
    if (self::hasAdminPermission($account)) {
      return TRUE;
    }

    if (!$storageController) {
      $storageController = \Drupal::service('entity_type.manager')->getStorage("taxonomy_term");
    }

    if (!empty($file->field_folder->entity)) {
      $folder = $file->field_folder->entity;
      if (self::hasFolderPermission('view', $folder, $account)) {
        return TRUE;
      }

      if (self::accessFolderRecursive($folder->id(), $account, $storageController)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Account has edit file permission.
   */
  public static function hasEditFileAccess($file, $account, $storageController = NULL) {
    if (self::hasAdminPermission($account)) {
      return TRUE;
    }

    if (!$storageController) {
      $storageController = \Drupal::service('entity_type.manager')->getStorage("taxonomy_term");
    }

    if ($account->hasPermission('access media folder file explorer') && !empty($file->field_folder->entity)) {
      $folder = $file->field_folder->entity;
      if (!self::accessFolderRecursive($folder->id(), $account, $storageController)) {
        return FALSE;
      }
      if ($file->field_locked->value) {
        return FALSE;
      }
      if (self::hasFolderPermission('edit_any', $folder, $account)) {
        return TRUE;
      }
      if ($account->id() == $file->uid->entity->id() && self::hasFolderPermission('edit_own', $folder, $account)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Account has delete file permission.
   */
  public static function hasDeleteFileAccess($file, $account, $storageController = NULL) {
    if (self::hasAdminPermission($account)) {
      return TRUE;
    }

    if (!$storageController) {
      $storageController = \Drupal::service('entity_type.manager')->getStorage("taxonomy_term");
    }

    if ($account->hasPermission('access media folder file explorer') && !empty($file->field_folder->entity)) {
      $folder = $file->field_folder->entity;
      if (!self::accessFolderRecursive($folder->id(), $account, $storageController)) {
        return FALSE;
      }
      if ($file->field_locked->value) {
        return FALSE;
      }
      if (self::hasFolderPermission('delete', $folder, $account)) {
        return TRUE;
      }
      if ($account->id() == $file->uid->entity->id() && self::hasFolderPermission('delete_own', $folder, $account)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Account has lock file permission.
   */
  public static function hasLockFileAccess($file, $account, $storageController = NULL) {
    if (self::hasAdminPermission($account)) {
      return TRUE;
    }

    if (!$storageController) {
      $storageController = \Drupal::service('entity_type.manager')->getStorage("taxonomy_term");
    }

    if ($account->hasPermission('access media folder file explorer') && !empty($file->field_folder->entity)) {
      $folder = $file->field_folder->entity;
      if (!self::accessFolderRecursive($folder->id(), $account, $storageController)) {
        return FALSE;
      }
      if (self::hasFolderPermission('lock', $folder, $account)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Account has file revisions permission.
   */
  public static function hasFileRevisionAccess($file, $account, $storageController) {
    if ($account->hasPermission('administer media folder files') || $account->hasPermission('administer all media folder files revisions') || $account->hasPermission('view all media folder files revisions')) {
      return TRUE;
    }
    elseif ($account->hasPermission('view own media folder files revisions') && ($account->id() == $file->uid->entity->id())) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Account has view folder permission.
   */
  public static function getUserViewFoldersAccess($folder, $account, $storageController) {
    $folder_access = [];
    $folders = $storageController->loadTree('media_folder', $folder, 1, TRUE);
    foreach ($folders as $sub_folder) {
      if (self::hasFolderPermission('view', $sub_folder, $account)) {
        $folder_access[] = $sub_folder->id();
        $folder_access = array_merge($folder_access, self::getUserViewFoldersAccess($sub_folder->id(), $account, $storageController));
      }
    }

    return $folder_access;
  }

  /**
   * Account has write folder permission.
   */
  public static function getUserWriteFoldersAccess($folder, $account, $storageController) {
    $folder_access = [];
    $folders = $storageController->loadTree('media_folder', $folder, 1, TRUE);
    foreach ($folders as $sub_folder) {
      if (self::hasFolderPermission('create', $sub_folder, $account)) {
        $folder_access[] = $sub_folder->id();
        $folder_access = array_merge($folder_access, self::getUserViewFoldersAccess($sub_folder->id(), $account, $storageController));
      }
    }

    return $folder_access;
  }

}
