<?php

namespace Drupal\media_folder_management\Entity\Access;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\media_folder_management\MediaFolderFileInterface;
use Symfony\Component\Routing\Route;

/**
 * Provides an access checker for media folder file item revisions.
 *
 * @ingroup media_folder_file_access
 */
class MediaFolderFileRevisionAccessCheck implements AccessInterface {

  /**
   * The media folder file storage.
   *
   * @var \Drupal\Core\Entity\ContentEntityStorageInterface
   */
  protected $mediaFolderFileStorage;

  /**
   * The media folder file access control handler.
   *
   * @var \Drupal\Core\Entity\EntityAccessControlHandlerInterface
   */
  protected $mediaFolderFileAccess;

  /**
   * A static cache of access checks.
   *
   * @var array
   */
  protected $access = [];

  /**
   * Constructs a new MediaFolderFileRevisionAccessCheck.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->mediaFolderFileStorage = $entity_type_manager->getStorage('media_folder_file');
    $this->mediaFolderFileAccess = $entity_type_manager->getAccessControlHandler('media_folder_file');
  }

  /**
   * Checks routing access for the media folder file item revision.
   *
   * @param \Symfony\Component\Routing\Route $route
   *   The route to check against.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The currently logged in account.
   * @param int $media_folder_file_revision
   *   The item revision ID. If not specified, but $media_folder_file is,
   *   access is checked for that object's revision.
   * @param \Drupal\media_folder_management\MediaFolderFileInterface|null $media_folder_file
   *   A media folder file item. Used for checking access to items
   *   default revision when $media_folder_file is unspecified. Ignored when
   *   is specified. If neither item revision nor item are
   *   specified, then access is denied.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(Route $route, AccountInterface $account, $media_folder_file_revision = NULL, ?MediaFolderFileInterface $media_folder_file = NULL) {
    if ($media_folder_file_revision) {
      $media_folder_file = $this->mediaFolderFileStorage->loadRevision($media_folder_file_revision);
    }
    $operation = $route->getRequirement('_access_media_folder_file_revision');
    return AccessResult::allowedIf($media_folder_file && $this->checkAccess($media_folder_file, $account, $operation))->cachePerPermissions()->addCacheableDependency($media_folder_file);
  }

  /**
   * Checks media folder file item revision access.
   *
   * @param \Drupal\media_folder_management\MediaFolderFileInterface $media_folder_file
   *   The media folder file item to check.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   A user object representing the user for whom the operation is to be
   *   performed.
   * @param string $op
   *   (optional) The specific operation being checked. Defaults to 'view'.
   *
   * @return bool
   *   TRUE if the operation may be performed, FALSE otherwise.
   */
  public function checkAccess(MediaFolderFileInterface $media_folder_file, AccountInterface $account, $op = 'view') {
    if (!$media_folder_file) {
      return FALSE;
    }

    $langcode = $media_folder_file->language()->getId();
    $cid = $media_folder_file->getRevisionId() . ':' . $langcode . ':' . $account->id() . ':' . $op;

    if (!isset($this->access[$cid])) {
      if ($account->hasPermission('administer media folder files') || $account->hasPermission('administer all media folder files revisions')) {
        $this->access[$cid] = TRUE;
      }
      else {
        switch ($op) {
          case 'view':
            if ($account->hasPermission('view all media folder files revisions')) {
              $this->access[$cid] = TRUE;
            }
            elseif ($account->hasPermission('view own media folder files revisions') && ($account->id() == $media_folder_file->uid->entity->id())) {
              $this->access[$cid] = TRUE;
            }
            else {
              $this->access[$cid] = FALSE;
            }

            break;

          case 'update':
            if ($account->hasPermission('administer own media folder files revisions') && ($account->id() == $media_folder_file->uid->entity->id())) {
              $this->access[$cid] = TRUE;
            }
            else {
              $this->access[$cid] = FALSE;
            }

            break;

          default:
            $this->access[$cid] = FALSE;
        }
      }
    }

    return $this->access[$cid];
  }

  /**
   * Counts the number of revisions in the default language.
   *
   * @param \Drupal\media_folder_management\MediaFolderFileInterface $media_folder_file
   *   The media folder file item for which to to count the revisions.
   *
   * @return int
   *   The number of revisions in the default language.
   */
  protected function countDefaultLanguageRevisions(MediaFolderFileInterface $media_folder_file) {
    $entity_type = $media_folder_file->getEntityType();
    $count = $this->mediaFolderFileStorage->getQuery()
      ->allRevisions()
      ->condition($entity_type->getKey('id'), $media_folder_file->id())
      ->count()
      ->accessCheck(FALSE)->execute();
    return $count;
  }

}
