<?php

namespace Drupal\media_folder_management\Entity\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\media_folder_management\Entity\Access\MediaFolderFileAccessCheck;
use Drupal\workflows\Entity\Workflow;

/**
 * Form controller for the media folder file edit forms.
 *
 * @internal
 */
class MediaFolderFileForm extends ContentEntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    $media_folder_file_type = $this->entity->bundle->entity;

    if ($this->operation === 'edit') {
      $form['#title'] = $this->t('Edit %type_label @label', [
        '%type_label' => $media_folder_file_type->label(),
        '@label' => $this->entity->label(),
      ]);
    }

    if (!$this->currentUser()->hasPermission('administer folders')) {
      $form['uid']['#access'] = FALSE;
      $form['uid']['widget'][0]['target_id']['#default_value'] = $this->entityTypeManager->getStorage('user')->load($this->currentUser()->id());
    }

    $form['created']['#access'] = FALSE;
    $form['revision_information']['#access'] = FALSE;
    unset($form['revision_log_message']['#group']);

    if ($this->operation == 'edit') {
      $folder = $this->entity->field_folder->entity->id();
      $storageController = $this->entityTypeManager->getStorage("taxonomy_term");
      $my_folders = MediaFolderFileAccessCheck::getUserWriteFoldersAccess(0, $this->currentUser(), $storageController);
      $form['field_folder']['widget']['#options'] = $this->geAllUserFoldersArray($storageController);
      foreach (array_keys($form['field_folder']['widget']['#options']) as $key) {
        if (!in_array($key, $my_folders)) {
          unset($form['field_folder']['widget']['#options'][$key]);
        }
      }
      $form['field_folder']['widget']['#default_value'] = $folder;
    }
    elseif ($this->operation == 'add') {
      $folder = $this->getRouteMatch()->getParameter('folder');
      $form['field_folder']['widget']['#default_value'] = $folder;
      $form['field_folder']['#access'] = FALSE;
    }

    $form['field_locked']['widget']['value'] = [
      '#type' => 'hidden',
      '#value' => $form['field_locked']['widget']['value']['#default_value'],
    ];

    $workflow = static::getMediaFolderFileWorkflow();
    if (!$workflow) {
      $form['status']['widget']['value'] = [
        '#type' => 'hidden',
        '#value' => $form['status']['widget']['value']['#default_value'],
      ];
    }

    $form['#attached']['library'][] = 'media_folder_management/folders';

    return $form;
  }

  /**
   * Get user folders.
   */
  public function geAllUserFoldersArray($storageController, $with_root = FALSE) {
    $folder_array = ($with_root) ? ['root' => $this->t('Root')] : [];
    $folders = $storageController->loadTree('media_folder', 0, NULL, FALSE);
    foreach ($folders as $folder) {
      $depth = ($with_root) ? ($folder->depth + 1) : $folder->depth;
      $folder_array[$folder->tid] = str_repeat('-', $depth) . ' ' . $folder->name;
    }

    return $folder_array;
  }

  /**
   * {@inheritdoc}
   */
  public static function getMediaFolderFileWorkflow() {
    if (\Drupal::service('module_handler')->moduleExists('content_moderation')) {
      $workflows = Workflow::loadMultipleByType('content_moderation');
      foreach ($workflows as $workflow) {
        $type_settings = $workflow->get('type_settings');
        if (array_key_exists('media_folder_file', $type_settings['entity_types'])) {
          return $workflow;
        }
      }
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $this->buildEntity($form, $form_state);
    $values = $form_state->getValues();

    _media_folder_management_validate_file_name($form_state, $values['field_folder'][0]['target_id'], $values['name'][0]['value'], $entity->lid->value);
    parent::validateForm($form, $form_state);

    return $entity;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $saved = parent::save($form, $form_state);
    $context = [
      '@type' => $this->entity->bundle(),
      '%label' => $this->entity->label(),
      'link' => $this->entity->toLink($this->t('View'))->toString(),
    ];
    $logger = $this->logger('media_folder_management');
    $t_args = [
      '@type' => $this->entity->bundle->entity->label(),
      '%label' => $this->entity->toLink($this->entity->label())->toString(),
    ];

    if ($saved === SAVED_NEW) {
      $logger->notice('@type: added %label.', $context);
      $this->messenger()->addStatus($this->t('@type %label has been created.', $t_args));
    }
    else {
      $logger->notice('@type: updated %label.', $context);
      $this->messenger()->addStatus($this->t('@type %label has been updated.', $t_args));
    }

    if ($this->currentUser()->hasPermission('administer folders')) {
      $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    }
    else {
      $form_state->setRedirectUrl($this->entity->toUrl());
    }

    return $saved;
  }

}
