<?php

namespace Drupal\media_folder_management\Entity\Form;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Datetime\DateFormatter;

/**
 * Provides a form for reverting a media folder file revision.
 *
 * @internal
 */
class MediaFolderFileRevisionDeleteForm extends ConfirmFormBase {

  /**
   * The media folder file revision.
   *
   * @var \Drupal\media_folder_management\MediaFolderFileInterface
   */
  protected $revision;

  /**
   * The media folder file storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $mediaFolderFileStorage;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityStorageInterface $media_folder_file_storage, Connection $connection, DateFormatter $date_formatter) {
    $this->mediaFolderFileStorage = $media_folder_file_storage;
    $this->connection = $connection;
    $this->dateFormatter = $date_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $entity_manager = $container->get('entity_type.manager');
    return new static(
      $entity_manager->getStorage('media_folder_file'),
      $container->get('database'),
      $container->get('date.formatter'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'media_folder_file_revision_delete_confirm';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to delete the revision from %revision-date?', ['%revision-date' => $this->dateFormatter->format($this->revision->getRevisionCreationTime())]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('entity.media_folder_file.version_history', ['media_folder_file' => $this->revision->id()]);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Delete');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $media_folder_file_revision = NULL) {
    $this->revision = $this->mediaFolderFileStorage->loadRevision($media_folder_file_revision);
    $form = parent::buildForm($form, $form_state);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->mediaFolderFileStorage->deleteRevision($this->revision->getRevisionId());

    $this->logger('media_folder_management')->notice('@type: deleted %title revision %revision.', [
      '@type' => $this->revision->bundle(),
      '%title' => $this->revision->label(),
      '%revision' => $this->revision->getRevisionId(),
    ]);
    $this->messenger()
      ->addStatus($this->t('Revision from %revision-date %title has been deleted.', [
        '%revision-date' => $this->dateFormatter->format($this->revision->getRevisionCreationTime()),
        '%title' => $this->revision->label(),
      ]));
    $form_state->setRedirect(
      'entity.media_folder_file.canonical',
      ['media_folder_file' => $this->revision->id()]
    );
    if ($this->connection->query('SELECT COUNT(DISTINCT vid) FROM {media_folder_file_revision} WHERE lid = :lid', [':lid' => $this->revision->id()])->fetchField() > 1) {
      $form_state->setRedirect(
        'entity.media_folder_file.version_history',
        ['media_folder_file' => $this->revision->id()]
      );
    }
  }

}
