<?php

namespace Drupal\media_folder_management\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form builder.
 */
class EditFolderForm extends FormBase {

  /**
   * The Get EntityTypeManagerInterface.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, AccountInterface $current_user) {
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $this->entityTypeManager->getStorage('user')->load($current_user->id());
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('entity_type.manager'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'add-folder-form';
  }

  /**
   * {@inheritdoc}
   */
  public function getFields() {
    $fields = [
      'field_perms_view' => $this->t('View/Download'),
      'field_perms_create' => $this->t('Create'),
      'field_perms_edit_any' => $this->t('Edit Any'),
      'field_perms_edit_own' => $this->t('Edit Own'),
      'field_perms_delete' => $this->t('Delete Any'),
      'field_perms_delete_own' => $this->t('Delete Own'),
      'field_perms_lock' => $this->t('Lock'),
    ];

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, mixed $folder = NULL) {
    $form['name'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Name'),
      '#default_value' => $folder->getName(),
      '#maxlength' => 40,
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#required' => FALSE,
      '#title' => $this->t('Description'),
      '#default_value' => (!empty($folder->description->value)) ? strip_tags($folder->description->value) : '',
    ];

    if ($this->currentUser->hasPermission('administer folders') || $this->currentUser->hasPermission('administer media folder files')) {
      $roles = $this->entityTypeManager->getStorage('user_role')->loadMultiple();
      $fields = $this->getFields();

      $form['inherit'] = [
        '#title' => $this->t('Inherit Permissions'),
        '#type' => 'checkbox',
        '#default_value' => (!empty($folder->field_perms_inherit->value) && $folder->field_perms_inherit->value) ? 1 : 0,
      ];

      $form['perms_wrapper'] = [
        '#type' => 'details',
        '#open' => TRUE,
        '#title' => $this->t('Permissions'),
        "#states" => [
          'visible' => [
            ':input[name="inherit"]' => ['checked' => FALSE],
          ],
        ],
      ];

      $form['perms_wrapper']['permissions'] = [
        '#type' => 'table',
        '#empty' => $this->t('No permissions found.'),
        '#header' => [$this->t('Role')],
        '#tree' => TRUE,
        '#title' => $this->t('Permissions'),
      ];

      foreach ($fields as $field => $name) {
        $field_values = $folder->$field->getValue();
        $form['perms_wrapper']['permissions']['#header'][] = [
          'data' => $name,
          'class' => ['checkbox'],
        ];
        foreach ($roles as $role) {
          $checked = FALSE;
          foreach ($field_values as $field_value) {
            if (in_array($role->id(), $field_value)) {
              $checked = TRUE;
            }
          }
          $form['perms_wrapper']['permissions'][$role->id()]['label'] = [
            '#markup' => $role->label(),
          ];
          $form['perms_wrapper']['permissions'][$role->id()][$field] = [
            '#title' => $role->label(),
            '#title_display' => 'invisible',
            '#wrapper_attributes' => [
              'class' => ['checkbox'],
            ],
            '#type' => 'checkbox',
            '#default_value' => $checked,
            '#parents' => [$field, $role->id()],
          ];
        }
      }
    }

    $form['folder'] = [
      '#type' => 'hidden',
      '#value' => $folder->id(),
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $term = $this->entityTypeManager->getStorage('taxonomy_term')->load($values['folder']);
    _media_folder_management_validate_folder_name($form_state, $term->parent->target_id, $values['name'], $term->id());
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    $term = $this->entityTypeManager->getStorage('taxonomy_term')->load($values['folder']);
    $term->name->value = $values['name'];
    $term->description->value = $values['description'];
    $term->field_perms_inherit->value = $values['inherit'];

    if ($values['inherit'] == FALSE) {
      $fields = $this->getFields();
      foreach ($fields as $key => $value) {
        $field = [];
        if (!empty($values[$key])) {
          foreach ($values[$key] as $role => $allowed) {
            if ($allowed) {
              $field[] = ['target_id' => $role];
            }
          }
        }
        $term->set($key, $field);
      }
    }

    $term->save();
    $this->messenger()->addStatus('Folder saved');
  }

}
