<?php

namespace Drupal\media_folder_management;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileExists;
use Drupal\Core\File\FileSystemInterface;
use Drupal\file\FileRepository;
use Drupal\file\Validation\FileValidator;
use Drupal\Core\File\FileSystem;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Utility\Token;
use Drupal\Core\Transliteration\PhpTransliteration;
use Drupal\Core\Language\LanguageManagerInterface;

/**
 * Service which builds the file explorer widget.
 */
class MediaFolderFileUiActions {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The config service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * File file repository object.
   *
   * @var \Drupal\file\FileRepository
   */
  protected $fileRepository;

  /**
   * File file validator object.
   *
   * @var \Drupal\file\Validation\FileValidator
   */
  protected $fileValidator;

  /**
   * File file system object.
   *
   * @var \Drupal\Core\File\FileSystem
   */
  protected $fileSystem;

  /**
   * File token object.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * File transliteration object.
   *
   * @var \Drupal\Core\Transliteration\PhpTransliteration
   */
  protected $transliteration;

  /**
   * The language service.
   *
   * @var \Drupal\Core\Language\LanguageInterface
   */
  protected $languageManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    ConfigFactoryInterface $config_factory,
    FileRepository $file_repository,
    FileValidator $file_validator,
    FileSystem $file_system,
    Token $token,
    PhpTransliteration $php_transliteration,
    LanguageManagerInterface $languageManager,
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
    $this->fileRepository = $file_repository;
    $this->fileValidator = $file_validator;
    $this->fileSystem = $file_system;
    $this->token = $token;
    $this->transliteration = $php_transliteration;
    $this->languageManager = $languageManager;
  }

  /**
   * Ajax upload files to folder.
   */
  public function ajaxUploadFile($folder, $request) {
    $response = [
      'messages' => [],
    ];

    if (!empty($request['request'])) {
      $field_config = $this->entityTypeManager->getStorage('field_config')->load('media_folder_file.file.field_file');
      $field_settings = $field_config->getSettings();
      $validators = [];
      if (!empty($field_settings['file_extensions'])) {
        $validators['FileExtension'] = ['extensions' => $field_settings['file_extensions']];
      }
      if (!empty($field_settings['max_filesize'])) {
        $validators['FileSizeLimit'] = ['fileLimit' => $field_settings['max_filesize']];
      }

      $filenames = [];
      $files = $this->entityTypeManager->getStorage('media_folder_file')->loadByProperties([
        'bundle' => 'file',
        'field_folder' => $folder->id(),
        'status' => TRUE,
      ]);
      if (!empty($files)) {
        foreach ($files as $file) {
          $filenames[$file->getName()] = $file->getName();
        }
      }

      foreach ($request['request'] as $post) {
        try {
          $media_file_name = $post['name'];
          if (in_array($media_file_name, $filenames)) {
            $pos = strrpos($media_file_name, '.');
            if ($pos !== FALSE) {
              $name = substr($media_file_name, 0, $pos);
              $ext = substr($media_file_name, $pos);
            }
            else {
              $name = $media_file_name;
              $ext = '';
            }

            $counter = 1;
            do {
              $media_file_name = $name . ' (' . $counter++ . ')' . $ext;
            } while (in_array($media_file_name, $filenames));
          }

          $files_data = preg_replace('#^data:application/\w+;base64,#i', '', $post['contents']);
          $files_data = preg_replace('#^data:image/\w+;base64,#i', '', $files_data);
          $files_data = preg_replace('#^data:video/\w+;base64,#i', '', $files_data);
          $files_data = preg_replace('#^data:audio/\w+;base64,#i', '', $files_data);
          $files_data = preg_replace('#^data:text/\w+;base64,#i', '', $files_data);
          $file_data = base64_decode($files_data);
          $file = $this->fileRepository->writeData($file_data, 'temporary://' . $this->sanitizeFilename($media_file_name), FileExists::Rename);
          $file->status->value = 0;
          $file->save();

          $invalid = $this->fileValidator->validate($file, $validators);
          if ($invalid->count() > 0) {
            for ($i = 0; $i < $invalid->count(); $i++) {
              $response['messages'][] = [
                'type' => 'error',
                'message' => $invalid->get($i)->getMessage()->render(),
              ];
            }
          }
          else {
            $destination = $this->token->replace($field_settings['uri_scheme'] . '://' . $field_settings['file_directory']);
            $this->fileSystem->prepareDirectory($destination, FileSystemInterface::CREATE_DIRECTORY);
            $file->status->value = 1;
            $file->save();
            $this->fileRepository->move($file, $destination, FileExists::Rename);

            $folder_file = $this->entityTypeManager->getStorage('media_folder_file')->create(
              [
                'bundle' => 'file',
                'name' => $media_file_name,
                'field_file' => [
                  'target_id' => $file->id(),
                ],
                'field_folder' => [
                  'target_id' => $folder->id(),
                ],
                'status' => 1,
              ]
            );
            $folder_file->save();

            $response['messages'][] = [
              'type' => 'status',
              'message' => $this->t('File @filename uploaded!', ['@filename' => $post['name']]),
            ];
          }
        }
        catch (\Exception $e) {
          $response['messages'][] = [
            'type' => 'error',
            'message' => $e->getMessage(),
          ];
        }
      }
    }

    return $response;
  }

  /**
   * Ajax upload files to folder.
   */
  public function ajaxMoveInto($folder, $request) {
    $response = [
      'messages' => [],
    ];

    if (!empty($request['objects'])) {
      foreach ($request['objects'] as $object) {
        if (!empty($object['type']) && !empty($object['mid'])) {
          if ($object['type'] == 'folder') {
            if ($object['mid'] == $folder->id()) {
              $response['messages'][] = [
                'type' => 'error',
                'message' => $this->t('Can not move folder inside itself!'),
              ];
            }
            else {
              $move = $this->entityTypeManager->getStorage('taxonomy_term')->load($object['mid']);
              if ($move) {
                $children = $this->entityTypeManager->getStorage('taxonomy_term')->getChildren($folder);
                if (!empty($children)) {
                  foreach ($children as $child) {
                    if (strtolower($child->getName()) == strtolower($move->getName())) {
                      $response['messages'][] = [
                        'type' => 'error',
                        'message' => $this->t('There is already one folder with the same name!'),
                      ];
                    }
                  }
                }
                $move->parent->target_id = $folder->id();
                $move->save();
                $response['messages'][] = [
                  'type' => 'status',
                  'message' => $this->t('Folder @folder moved!', ['@folder' => $move->getName()]),
                ];
              }
            }
          }
          elseif ($object['type'] == 'file') {
            $move = $this->entityTypeManager->getStorage('media_folder_file')->load($object['mid']);
            if ($move) {
              $files = $this->entityTypeManager->getStorage('media_folder_file')->loadByProperties([
                'bundle' => 'file',
                'field_folder' => $folder->id(),
                'status' => TRUE,
              ]);
              if (!empty($files)) {
                foreach ($files as $file) {
                  if (strtolower($file->getName()) == strtolower($move->getName())) {
                    $response['messages'][] = [
                      'type' => 'error',
                      'message' => $this->t('There is already one file with the same name!'),
                    ];
                  }
                }
              }
              $move->field_folder->target_id = $folder->id();
              $move->save();
              $response['messages'][] = [
                'type' => 'status',
                'message' => $this->t('File @filename moved!', ['@filename' => $move->getName()]),
              ];
            }
          }
        }
      }
    }

    return $response;
  }

  /**
   * Sanitize filename.
   */
  public function sanitizeFilename($filename) {
    $fileSettings = $this->configFactory->get('file.settings');
    $transliterate = $fileSettings->get('filename_sanitization.transliterate');
    $extension = pathinfo($filename, PATHINFO_EXTENSION);
    if ($extension !== '') {
      $extension = '.' . $extension;
      $filename = pathinfo($filename, PATHINFO_FILENAME);
    }

    $alphanumeric = $fileSettings->get('filename_sanitization.replace_non_alphanumeric');
    $replacement = $fileSettings->get('filename_sanitization.replacement_character');
    if ($transliterate) {
      $transliterated_filename = $this->transliteration->transliterate(
        $filename,
        $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId(),
        $replacement
      );
      if (mb_strlen($transliterated_filename) > 0) {
        $filename = $transliterated_filename;
      }
      else {
        $alphanumeric = TRUE;
      }
    }
    if ($fileSettings->get('filename_sanitization.replace_whitespace')) {
      $filename = preg_replace('/\s/u', $replacement, trim($filename));
    }
    if ($transliterate && $alphanumeric) {
      $filename = preg_replace('/[^0-9A-Za-z_.-]/u', $replacement, $filename);
    }
    if ($fileSettings->get('filename_sanitization.deduplicate_separators')) {
      $filename = preg_replace('/(_)_+|(\.)\.+|(-)-+/u', $replacement, $filename);
      $filename = preg_replace('/(_|\.|\-)[(_|\.|\-)]+/u', $replacement, $filename);
      $filename = preg_replace('/' . preg_quote($replacement, NULL) . '[' . preg_quote($replacement, NULL) . ']*/u', $replacement, $filename);
      $filename = rtrim($filename, $replacement);

      if (!empty($extension)) {
        $filename = rtrim($filename, '.');
      }
    }
    if ($fileSettings->get('filename_sanitization.lowercase')) {
      $filename = mb_strtolower($filename);
    }

    return $filename . $extension;
  }

}
