<?php

namespace Drupal\media_folder_management;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\SortArray;
use Drupal\Component\Utility\Unicode;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Markup;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\media_folder_management\Entity\Access\MediaFolderFileAccessCheck;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Routing\RedirectDestinationInterface;
use Drupal\Core\File\FileUrlGenerator;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\media_folder_management\Entity\MediaFolderFile;

/**
 * Service which builds the file explorer widget.
 */
class MediaFolderFileUiBuilder {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The config service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The currently active request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * Current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The redirect destination service.
   *
   * @var \Drupal\Core\Routing\RedirectDestinationInterface
   */
  protected $redirectDestination;

  /**
   * The default cache.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * File url generator object.
   *
   * @var \Drupal\Core\File\FileUrlGenerator
   */
  protected $fileUrlGenerator;

  /**
   * Drupal\Core\Render\RendererInterface definition.
   *
   * @var Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Form builder will be used via Dependency Injection.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * The media folder file settings.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $mediaFolderFileSettings;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    ConfigFactoryInterface $config_factory,
    RouteMatchInterface $route_match,
    RequestStack $request_stack,
    AccountInterface $current_user,
    RedirectDestinationInterface $redirect_destination,
    CacheBackendInterface $cache,
    FileUrlGenerator $fileUrlGenerator,
    RendererInterface $renderer,
    FormBuilderInterface $form_builder,
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
    $this->request = $request_stack->getCurrentRequest();
    $this->currentUser = $current_user;
    $this->routeMatch = $route_match;
    $this->redirectDestination = $redirect_destination;
    $this->cache = $cache;
    $this->fileUrlGenerator = $fileUrlGenerator;
    $this->renderer = $renderer;
    $this->formBuilder = $form_builder;
    $this->mediaFolderFileSettings = $config_factory->get('media_folder_management.settings');
  }

  /**
   * {@inheritdoc}
   */
  public static function dialogOptions() {
    return [
      'classes' => [
        'ui-dialog' => 'media-folders-widget-modal',
      ],
      'title' => t('Add or select files'),
      'height' => '75%',
      'width' => '75%',
    ];
  }

  /**
   * Get title.
   */
  public function getTitle(mixed $folder = NULL) {
    if (!$folder) {
      return $this->mediaFolderFileSettings->get('page_name');
    }
    $element = ($folder && ($folder->bundle() == 'media_folder')) ? $folder->getName() : '';

    return $this->mediaFolderFileSettings->get('page_name') . ' | ' . $element;
  }

  /**
   * {@inheritdoc}
   */
  public function buildAdminUi($folder = NULL) {
    $folder_id = ($folder) ? $folder->id() : 0;
    $view_type = $this->getCookie('view-type', $this->mediaFolderFileSettings->get('default_view'));

    $folders = $this->getFolderFolders($folder_id, $view_type);
    $folders_ordered = $this->itemsOrder($folders);
    $this->getFolderFiles($folder, $folders_ordered);
    $this->addTableHeader($folders_ordered);
    $navbar = $this->geAllUserFoldersTree($folder_id);
    $actions = $this->getFolderActions($folder);
    $actions_raw = $this->getRawActions($actions);
    $subfolders_class = (!$folder || !MediaFolderFileAccessCheck::hasFolderPermission('create', $folder, $this->currentUser)) ? 'not-droppable' : '';

    $table = [
      '#theme' => 'media_folder_page',
      '#title' => $this->getTitle($folder),
      '#up_button' => $this->getUpButton($folder),
      '#breadcrumb' => $this->getBreadcrumb($folder),
      '#actions' => $actions,
      '#actions_raw' => ($actions_raw) ? $actions_raw : NULL,
      '#buttons' => Markup::create($this->getButtons()),
      '#form' => Markup::create($this->getForm()),
      '#folders' => [
        '#theme' => 'media_folder_folders',
        '#folders' => $folders_ordered,
      ],
      '#navbar' => [
        '#theme' => 'media_folder_navbar',
        '#folders' => $navbar,
      ],
      '#class' => $view_type,
      '#subfolders_class' => $subfolders_class,
      '#fid' => $folder_id,
      '#attached' => [
        'library' => 'media_folder_management/folders',
      ],
      '#cache' => [
        'tags' => [
          'media_folder_page',
          'media_folder_page:' . $folder_id,
        ],
        'contexts' => [
          'user.roles',
        ],
      ],
    ];

    return $table;
  }

  /**
   * {@inheritdoc}
   */
  public function buildAjaxResponse($folder, &$build) {
    $folder_id = ($folder) ? $folder->id() : 0;
    $view_type = $this->getCookie('view-type', $this->mediaFolderFileSettings->get('default_view'));

    $folders = $this->getFolderFolders($folder_id, $view_type);
    $folders_ordered = $this->itemsOrder($folders);
    $this->getFolderFiles($folder, $folders_ordered);
    $this->addTableHeader($folders_ordered);
    $actions = $this->getFolderActions($folder);
    $actions_raw = $this->getRawActions($actions);
    $up = $this->getUpButton($folder);
    $folders = [
      '#theme' => 'media_folder_folders',
      '#folders' => $folders_ordered,
    ];
    $navbar = [
      '#theme' => 'media_folder_navbar',
      '#folders' => $this->geAllUserFoldersTree($folder_id),
    ];
    $subfolders_class = (!$folder || !MediaFolderFileAccessCheck::hasFolderPermission('create', $folder, $this->currentUser)) ? 'not-droppable' : '';

    $response = [
      'navbar' => '<div class="navbar-header">' . $this->t('Folders') . '</div>' . $this->renderer->render($navbar),
      'folders' => $this->renderer->render($folders) . '<div class="droppable-message">' . $this->t('Drop your file here!') . '</div>',
      'breadcrumb' => $this->getBreadcrumb($folder),
      'actions' => $this->renderer->render($actions),
      'actions_raw' => ($actions_raw) ? '<span class="close"></span>' . $this->renderer->render($actions_raw) : '',
      'buttons' => $this->getButtons(),
      'up_button' => (!$folder) ? $up : $this->renderer->render($up),
      'title' => $this->getTitle($folder),
      'subfolders_class' => $subfolders_class,
      'fid' => $folder_id,
    ];

    $build['#cache'] = [
      'tags' => [
        'media_folder_page:' . $folder_id,
      ],
      'contexts' => ['user.roles'],
    ];

    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function buildWidgetUi($state = NULL) {
    if (!$state) {
      $state = MediaFolderFileState::fromRequest($this->request);
    }

    $widget_settings = $state->getWidgetSettings();
    $opener_parameters = $state->getOpenerParameters();
    $folder_id = 0;
    $view_type = $widget_settings['view_type'];

    $folders = $this->getFolderFolders($folder_id, $view_type, FALSE);
    $folders_ordered = $this->itemsOrder($folders, $widget_settings);
    $navbar = $this->geAllUserFoldersTree($folder_id);
    $subfolders_class = 'not-droppable';

    $explorer = [
      '#theme' => 'media_folder_widget',
      '#up_button' => ($widget_settings['show_navigation']) ? $this->getUpButton(NULL) : NULL,
      '#breadcrumb' => ($widget_settings['show_navigation']) ? $this->getBreadcrumb(NULL) : NULL,
      '#form' => ($widget_settings['show_search']) ? Markup::create($this->getForm()) : NULL,
      '#folders' => [
        '#theme' => 'media_folder_folders',
        '#folders' => $folders_ordered,
      ],
      '#navbar' => [
        '#theme' => 'media_folder_navbar',
        '#folders' => $navbar,
      ],
      '#class' => 'widget ' . $view_type,
      '#subfolders_class' => $subfolders_class,
      '#fid' => $folder_id,
      '#cache' => [
        'tags' => [
          'media_folder_page:' . $folder_id,
        ],
        'contexts' => [
          'user.roles',
        ],
      ],
      '#attached' => [
        'drupalSettings' => [
          'media_folder_management' => [
            'selection_remaining' => $state->getAvailableSlots(),
            'state_parameters' => $opener_parameters,
            'widget_settings' => $widget_settings,
          ],
        ],
      ],
    ];

    return $explorer;
  }

  /**
   * {@inheritdoc}
   */
  public function buildWidgetAjaxResponse($folder, $request, $entity_type_id, $bundle, $widget_id, &$build) {
    $folder_id = ($folder) ? $folder->id() : 0;
    $widget_settings = $request['request'];
    $view_type = $widget_settings['view_type'];
    $opener_parameters = [
      'entity_type_id' => $entity_type_id,
      'bundle' => $bundle,
      'field_name' => $widget_id,
    ];

    $folders = $this->getFolderFolders($folder_id, $view_type, FALSE);
    $folders_ordered = $this->itemsOrder($folders, $widget_settings);
    $this->getWidgetFolderFiles($folder, $folders_ordered, $opener_parameters, $widget_settings);
    $this->addTableHeader($folders_ordered);

    $folders = [
      '#theme' => 'media_folder_folders',
      '#folders' => $folders_ordered,
    ];
    $navbar = [
      '#theme' => 'media_folder_navbar',
      '#folders' => $this->geAllUserFoldersTree($folder_id),
    ];
    $subfolders_class = 'not-droppable';
    $up = $this->getUpButton($folder);
    $up = $this->renderer->render($up);

    $response = [
      'navbar' => $this->renderer->render($navbar),
      'folders' => $this->renderer->render($folders),
      'subfolders_class' => $subfolders_class,
      'up_button' => ($widget_settings['show_navigation']) ? $up : NULL,
      'breadcrumb' => ($widget_settings['show_navigation']) ? $this->getBreadcrumb($folder) : NULL,
      'form' => ($widget_settings['show_search']) ? Markup::create($this->getForm()) : NULL,
      'fid' => $folder_id,
    ];

    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function fileExplorerPerms($folder) {
    $roles = $values = $header = $rows = [];

    $rolesObj = $this->entityTypeManager->getStorage('user_role')->loadMultiple();
    foreach ($rolesObj as $roleObj) {
      $roles[] = $roleObj->id();
    }

    $fields = [
      'field_perms_create' => $this->t('Create'),
      'field_perms_edit_own' => $this->t('Edit Own'),
      'field_perms_edit_any' => $this->t('Edit Any'),
      'field_perms_view' => $this->t('View/Download'),
      'field_perms_lock' => $this->t('Lock'),
      'field_perms_delete_own' => $this->t('Delete Own'),
      'field_perms_delete' => $this->t('Delete'),
    ];

    $header['role'] = $this->t('Role');
    foreach ($fields as $field => $label) {
      $header[$field] = $label;

      if (!empty($folder->parent->entity) && $folder->field_perms_inherit->value) {
        $title = $this->t('Folder Permissions for "@name", inherited from parent', ['@name' => $folder->getName()]);
      }
      else {
        $title = $this->t('Folder Permissions for "@name"', ['@name' => $folder->getName()]);
      }

      $values[$field] = MediaFolderFileAccessCheck::getParentPermissionTarget($folder, $field);
    }

    foreach ($roles as $role) {
      $row = [];
      $row['role'] = $role;
      foreach ($values as $field => $field_values) {
        foreach ($field_values as $field_value) {
          if (!empty($field_value['target_id']) && $field_value['target_id'] == $role) {
            $row[$field] = 'X';
            break;
          }
          else {
            $row[$field] = '-';
          }
        }
      }
      $rows[] = $row;
    }

    return [
      '#theme' => 'table',
      '#caption' => $title,
      '#header' => $header,
      '#rows' => $rows,
      '#attributes' => ['id' => 'perms-table'],
      '#cache' => [
        'tags' => [
          'media_folder_page',
          'media_folder_page:' . $folder->id(),
        ],
        'contexts' => [
          'user',
        ],
      ],
      '#attached' => [
        'library' => ['media_folder_management/folders'],
      ],
    ];
  }

  /**
   * Get cookie.
   */
  private function getCookie($name, $default) {
    $cookies = $this->request->cookies;
    if ($cookies->has($name)) {
      return $cookies->get($name);
    }

    return $default;
  }

  /**
   * Order items.
   */
  private function itemsOrder($items, $widget_settings = NULL) {
    $view_order = ($widget_settings) ? $widget_settings['view_order'] : $this->getCookie('view-order', $this->mediaFolderFileSettings->get('default_order'));

    if ($view_order == 'za' || $view_order == 'az') {
      uasort($items, [SortArray::class, 'sortByTitleProperty']);
    }
    elseif ($view_order == 'date-asc' || $view_order == 'date-desc') {
      uasort($items, [SortArray::class, 'sortByWeightProperty']);
    }

    if ($view_order == 'za' || $view_order == 'date-asc') {
      return array_reverse($items);
    }

    return $items;
  }

  /**
   * Get Folder Files.
   */
  private function getFolderFolders($folder_id, $view_type, $show_action = TRUE) {
    $folders = [];
    $tree = $this->entityTypeManager->getStorage('taxonomy_term')->loadTree('media_folder', $folder_id, 1, TRUE);
    $folder_route = $this->getFolderRoute($this->routeMatch->getRouteName());
    foreach ($tree as $term) {
      $classes = [
        'folder-icon',
        'folder-icon-folder-empty',
      ];

      if (!MediaFolderFileAccessCheck::hasFolderPermission('view', $term, $this->currentUser)) {
        continue;
      }
      $wrapper_classes = ['folder-folder'];
      if (!MediaFolderFileAccessCheck::hasFolderPermission('create', $term, $this->currentUser)) {
        $wrapper_classes[] = 'not-droppable';
      }

      $file_count = $this->getFileCount($term->id());

      $folders[] = [
        '#theme' => 'media_folder_folder',
        '#title' => $term->getName(),
        '#weight' => $term->changed->value,
        '#desc' => (!empty($term->getDescription())) ? Markup::create(strip_tags($term->getDescription())) : '',
        '#link' => Url::fromRoute($folder_route, ['folder' => $term->id()]),
        '#classes' => implode(' ', $classes),
        '#wrapper_classes' => implode(' ', $wrapper_classes),
        '#attr' => Markup::create('data-count="' . $file_count . '"'),
        '#size' => ($view_type == 'list') ? $file_count . ' ' . $this->t('items') : '',
        '#ftype' => ($view_type == 'list') ? $this->t('folder') : '',
        '#actions' => ($show_action) ? Markup::create($this->getItemActions($term)) : '',
        '#fid' => $term->id(),
        '#cache' => [
          'tags' => [
            'media_folder_item:' . $term->id(),
          ],
          'contexts' => [
            'user',
          ],
        ],
      ];
    }

    return $folders;
  }

  /**
   * Get Folder Files.
   */
  private function getFolderFiles($folder, &$folders) {
    if ($folder) {
      $files = [];
      $view_type = $this->getCookie('view-type', $this->mediaFolderFileSettings->get('default_view'));

      $values = $this->getFolderEntities($folder);
      foreach ($values as $file_entity) {
        if (!empty($file_entity->field_file->entity)) {
          $file = $file_entity->field_file->entity;
          $ext = pathinfo($file->getFilename(), PATHINFO_EXTENSION);
          $type = $this->humanFilesize($file->getSize());

          $files[] = [
            '#theme' => 'media_folder_folder',
            '#title' => $file_entity->getName(),
            '#weight' => $file_entity->field_file->entity->changed->value,
            '#desc' => $file_entity->field_file->description,
            '#link' => $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri()),
            '#classes' => $file_entity->getFileClasses(),
            '#wrapper_classes' => 'folder-file',
            '#attr' => Markup::create('download data-ext=".' . $ext . '" data-size="' . $type . '"' . $this->getLockedInfo($file_entity)),
            '#size' => ($view_type == 'list') ? $type : '',
            '#ftype' => ($view_type == 'list') ? $ext : '',
            '#actions' => Markup::create($this->getItemActions(FALSE, $file_entity)),
            '#fid' => $file_entity->id(),
            '#cache' => [
              'tags' => [
                'media_folder_item:' . $file_entity->id(),
              ],
              'contexts' => [
                'user',
              ],
            ],
          ];
        }
      }
      $files_ordered = $this->itemsOrder($files);

      $folders = array_merge($folders, $files_ordered);
    }
  }

  /**
   * Get Widget Folder Files.
   */
  private function getWidgetFolderFiles($folder, &$folders, $opener_parameters, $widget_settings) {
    if ($folder) {
      $show = array_values($widget_settings['show']);
      $allowed_extensions = $this->getWidgetAllowedExtensions($show);
      $view_type = $widget_settings['view_type'];

      $files = [];
      $values = $this->getFolderEntities($folder);
      foreach ($values as $file_entity) {
        if (!empty($file_entity->field_file->entity)) {
          $ext = $file_entity->getFileExtension();
          $type = $this->humanFilesize($file_entity->field_file->entity->getSize());

          if (!in_array($ext, $allowed_extensions)) {
            continue;
          }
          $files[] = [
            '#theme' => 'media_folder_folder',
            '#title' => $file_entity->getName(),
            '#weight' => $file_entity->field_file->entity->changed->value,
            '#desc' => $file_entity->field_file->description,
            '#link' => Url::fromRoute('media_folder_management.file.update_widget.ajax', [
              'folder' => $folder->id(),
              'media_folder_file' => $file_entity->id(),
              'widget_id' => $opener_parameters['field_name'],
            ])->toString(),
            '#classes' => $file_entity->getFileClasses(),
            '#wrapper_classes' => 'folder-file',
            '#attr' => '',
            '#size' => ($view_type == 'list') ? $type : '',
            '#ftype' => ($view_type == 'list') ? $ext : '',
            '#actions' => '',
            '#fid' => $file_entity->id(),
            '#cache' => [
              'tags' => [
                'media_folder_item:' . $file_entity->id(),
              ],
              'contexts' => [
                'user',
              ],
            ],
          ];
        }
      }
      $files_ordered = $this->itemsOrder($files, $widget_settings);

      $folders = array_merge($folders, $files_ordered);
    }
  }

  /**
   * Get Folder Entities.
   */
  private function getFolderEntities($folder) {
    $cid = 'media_folder_file:files:' . $folder->id();
    if ($cache = $this->cache->get($cid)) {
      $response = $cache->data;
    }
    else {
      $query = $this->entityTypeManager->getStorage('media_folder_file')->getQuery();
      $query->condition('bundle', 'file');
      $query->condition('field_folder', $folder->id());
      $query->condition('status', 1);
      $entity_ids = $query->accessCheck(FALSE)->execute();

      if (!empty($entity_ids)) {
        $response = $this->entityTypeManager->getStorage('media_folder_file')->loadMultiple($entity_ids);
      }
      else {
        $response = [];
      }
      $this->cache->set($cid, $response, CacheBackendInterface::CACHE_PERMANENT, [
        'taxonomy_term_list:media_folder',
        'media_folder_file_list:file',
      ]);
    }

    return $response;
  }

  /**
   * Get Widget Allowed Extensions.
   */
  private function getWidgetAllowedExtensions($show) {
    $allowed_extensions = [];
    if (in_array('images', $show)) {
      $allowed_extensions = array_merge($allowed_extensions, MediaFolderFile::getImageExtensions());
    }
    if (in_array('videos', $show)) {
      $allowed_extensions = array_merge($allowed_extensions, MediaFolderFile::getVideoExtensions());
    }
    if (in_array('audio', $show)) {
      $allowed_extensions = array_merge($allowed_extensions, MediaFolderFile::getAudioExtensions());
    }
    if (in_array('documents', $show)) {
      $allowed_extensions = array_merge($allowed_extensions, MediaFolderFile::getDocumentsExtensions());
    }
    if (in_array('other', $show)) {
      $allowed_extensions = array_merge($allowed_extensions, MediaFolderFile::getOtherExtensions());
    }

    return $allowed_extensions;
  }

  /**
   * Get File Count.
   */
  private function getFileCount($folder) {
    $sub_folder_count = $this->entityTypeManager->getStorage('taxonomy_term')->loadChildren($folder);
    $query = $this->entityTypeManager->getStorage('media_folder_file')->getQuery();
    $query->condition('bundle', 'file');
    $query->condition('field_folder', $folder);
    $query->condition('status', 1);
    $entity_ids = $query->accessCheck(FALSE)->execute();

    return count($sub_folder_count) + count($entity_ids);
  }

  /**
   * Add table header.
   */
  private function addTableHeader(&$folders) {
    $header = [
      '#theme' => 'media_folder_folder',
      '#title' => $this->t('Name'),
      '#weight' => -1,
      '#desc' => '',
      '#link' => '',
      '#classes' => '',
      '#wrapper_classes' => 'table-header',
      '#attr' => '',
      '#size' => $this->t('Size'),
      '#ftype' => $this->t('Type'),
      '#actions' => '',
      '#fid' => '',
    ];
    array_unshift($folders, $header);
  }

  /**
   * {@inheritdoc}
   */
  public function treeNestedArray(&$build, $tree, $my_folders, $folder_id, $route_name) {
    $folder_route = $this->getFolderRoute($route_name);

    foreach ($tree as $term) {
      if (!in_array($term->id(), $my_folders)) {
        continue;
      }
      $key = strtolower($term->getName()) . '-' . $term->id();
      $link = Link::fromTextAndUrl($term->getName(), Url::fromRoute($folder_route, ['folder' => $term->id()]));

      $folder_class = ['navbar-folder'];
      if (!MediaFolderFileAccessCheck::hasFolderPermission('create', $term, $this->currentUser)) {
        $folder_class[] = 'not-droppable';
      }

      $build[$key] = [
        'id' => $term->id(),
        'link' => $link,
        'class' => ($folder_id == $term->id()) ? 'active' : '',
        'folder_class' => implode(' ', $folder_class),
        'children' => [],
      ];
      $children = $this->entityTypeManager->getStorage('taxonomy_term')->getChildren($term);
      if (!empty($children)) {
        $this->treeNestedArray($build[$key]['children'], $children, $my_folders, $folder_id, $route_name);
      }
    }
    ksort($build);
  }

  /**
   * {@inheritdoc}
   */
  public function treeNestedArrayActiveTrail(&$build, $folder_id, $paths = FALSE) {
    if ($paths === FALSE) {
      $paths = $this->treeNestedArrayActiveTrailPaths($build, $folder_id);
    }

    if (!empty($paths)) {
      foreach ($paths as $path) {
        if (!empty($build[$path])) {
          $build[$path]['class'] .= ' active-trail';
          $this->treeNestedArrayActiveTrail($build[$path]['children'], $folder_id, $paths);
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function treeNestedArrayActiveTrailPaths($build, $folder_id) {
    $paths = [];
    $iterator = new \RecursiveIteratorIterator(new \RecursiveArrayIterator($build), \RecursiveIteratorIterator::SELF_FIRST);
    foreach ($iterator as $v) {
      if (is_array($v) && !empty($v['id']) && $v['id'] == $folder_id) {
        for ($p = [], $i = 0, $z = $iterator->getDepth(); $i <= $z; $i++) {
          $p = $iterator->getSubIterator($i)->key();
          if ($p != 'children') {
            $paths[] = $p;
          }
        }
      }
    }

    return $paths;
  }

  /**
   * {@inheritdoc}
   */
  public function geAllUserFoldersTree($folder_id) {
    $route_name = $this->routeMatch->getRouteName();
    $folders = $this->entityTypeManager->getStorage('taxonomy_term')->loadTree('media_folder', 0, 1, TRUE);
    $my_folders = MediaFolderFileAccessCheck::getUserViewFoldersAccess(0, $this->currentUser, $this->entityTypeManager->getStorage('taxonomy_term'));
    $build = [];
    $this->treeNestedArray($build, $folders, $my_folders, $folder_id, $route_name);
    $this->treeNestedArrayActiveTrail($build, $folder_id);

    return $build;
  }

  /**
   * Get folder actions.
   */
  private function getFolderActions($folder) {
    $action_links = [];

    $destination = $this->redirectDestination->getAsArray();
    if (!empty($destination['destination'])) {
      $url = parse_url($destination['destination']);
      $destination['destination'] = str_replace('/ajax', '', $url['path']);
    }

    if (!$folder && MediaFolderFileAccessCheck::hasAdminFolderPermission($this->currentUser)) {
      $action_links['new'] = [
        'title' => $this->t('New folder'),
        'url' => Url::fromRoute('media_folder_management.add_root_folder', [], ['query' => $destination]),
        'class' => ['new-folder'],
      ];
      $this->transformDialogLink($action_links['new'], $this->t('New folder'), TRUE, '90%');
    }
    elseif ($folder && (MediaFolderFileAccessCheck::hasAdminFolderPermission($this->currentUser) || MediaFolderFileAccessCheck::hasCreateFolderPermission($folder, $this->currentUser, TRUE))) {
      $action_links['new'] = [
        'title' => $this->t('New folder'),
        'url' => Url::fromRoute('media_folder_management.add_folder', ['folder' => $folder->id()], ['query' => $destination]),
        'class' => ['new-folder'],
      ];
      $this->transformDialogLink($action_links['new'], $this->t('New folder'), TRUE, '90%');
    }

    if ($folder) {
      if (MediaFolderFileAccessCheck::hasFolderPermission('create', $folder, $this->currentUser)) {
        $action_links['upload'] = [
          'title' => $this->t('Upload file'),
          'url' => Url::fromRoute('entity.media_folder_file.add_form', [
            'entity_type_id' => 'media_folder_file',
            'bundle_parameter' => 'media_folder_file_type',
            'media_folder_file_type' => 'file',
            'folder' => $folder->id(),
          ],
          ['query' => $destination]),
          'class' => ['upload-file'],
        ];
        $this->transformDialogLink($action_links['upload'], $this->t('Upload file'), TRUE, '70%');
      }

      if (MediaFolderFileAccessCheck::hasEditFolderPermission($folder, $this->currentUser, TRUE)) {
        $action_links['edit'] = [
          'title' => $this->t('Edit folder'),
          'url' => Url::fromRoute('media_folder_management.edit_folder', ['folder' => $folder->id()], ['query' => $destination]),
          'class' => ['edit-action'],
        ];
        $this->transformDialogLink($action_links['edit'], $this->t('Edit folder'), TRUE, '70%');
      }

      if (MediaFolderFileAccessCheck::hasDeleteFolderPermission($folder, $this->currentUser, TRUE)) {
        $action_links['delete'] = [
          'title' => $this->t('Delete folder'),
          'url' => Url::fromRoute('media_folder_management.delete_folder', ['folder' => $folder->id()]),
          'class' => ['delete-action'],
        ];
        $this->transformDialogLink($action_links['delete'], $this->t('Delete folder'), TRUE, '70%');
      }

      if (MediaFolderFileAccessCheck::hasViewFolderPermission($folder, $this->currentUser, TRUE)) {
        $action_links['share'] = [
          'title' => $this->t('Share link'),
          'url' => Url::fromRoute('media_folder_management.share', ['folder' => $folder->id()], ['query' => $destination]),
          'class' => ['share-action'],
        ];
        $this->transformDialogLink($action_links['share'], $this->t('Share link'), TRUE, '70%');
      }

      if (MediaFolderFileAccessCheck::hasAdminFolderPermission($this->currentUser)) {
        $action_links['perms'] = [
          'title' => $this->t('Permissions'),
          'url' => Url::fromRoute('media_folder_management.folder_perms', ['folder' => $folder->id()], ['query' => $destination]),
          'class' => ['perms-action'],
        ];
        $this->transformDialogLink($action_links['perms'], $this->t('Permissions'), TRUE, '70%');
      }
    }
    elseif (MediaFolderFileAccessCheck::hasAdminFolderPermission($this->currentUser)) {
      $action_links['manage'] = [
        'title' => $this->t('Manage folders'),
        'url' => Url::fromRoute('entity.taxonomy_vocabulary.overview_form', ['taxonomy_vocabulary' => 'media_folder'], ['query' => $destination]),
        'class' => ['manage-folder'],
      ];
      $this->transformDialogLink($action_links['manage'], $this->t('Manage folders'), TRUE);
    }

    if (!empty($action_links)) {
      $actions = [
        '#type' => 'dropbutton',
        '#links' => $action_links,
      ];

      return $actions;
    }

    return '';
  }

  /**
   * Create Link.
   */
  private function createLink($text, $url, $classes = [], $title = FALSE, $query = FALSE) {
    $link = Link::fromTextAndUrl($text, $url)->toRenderable();
    $link['#attributes'] = [
      'class' => $classes,
      'title' => ($title ? $title : $text),
    ];
    if ($query) {
      $destination = $this->redirectDestination->getAsArray();
      if (!empty($destination['destination'])) {
        $url = parse_url($destination['destination']);
        $destination['destination'] = str_replace('/ajax', '', $url['path']);
      }
      $link['#url']->setOption('query', $destination);
    }

    return $link;
  }

  /**
   * Create Dialog Link.
   */
  private function createDialogLink($text, $url, $classes = [], $title = FALSE, $query = FALSE, $width = '40%') {
    $link = $this->createLink($text, $url, $classes, $title, $query);
    $link['#attributes']['class'][] = 'use-ajax';
    $link['#attributes']['data-dialog-type'] = 'modal';
    $link['#attributes']['data-dialog-options'] = Json::encode(['width' => $width]);
    $link['#attached'] = [
      'library' => [
        'core/drupal.dialog.ajax',
      ],
    ];
    return $link;
  }

  /**
   * Transform Dialog Link.
   */
  private function transformDialogLink(&$link, $title = FALSE, $query = FALSE, $width = '40%') {
    $link['attributes']['class'][] = 'use-ajax';
    $link['attributes']['data-dialog-type'] = 'modal';
    $link['attributes']['data-dialog-options'] = Json::encode(['width' => $width]);
    $link['#attached'] = [
      'library' => [
        'core/drupal.dialog.ajax',
      ],
    ];
  }

  /**
   * Get Raw actions.
   */
  public function getRawActions($actions) {
    $actions_raw = [];
    if (!empty($actions['#links'])) {
      foreach ($actions['#links'] as $key => $link) {
        $actions_raw[$key] = [
          '#type'  => 'link',
          '#title' => $link['title'],
          '#url'  => $link['url'],
          '#attributes' => $link['attributes'],
          '#attached' => $link['#attached'],
        ];
        $actions_raw[$key]['#attributes']['class'][] = $link['class'][0];
      }
    }

    return $actions_raw;
  }

  /**
   * Get Up Button.
   */
  public function getUpButton($parent) {
    if (isset($parent->parent->target_id) && $parent->parent->target_id == 0) {
      if (str_starts_with($this->routeMatch->getRouteName(), 'media_folder_management.collection')) {
        $route = 'media_folder_management.collection';
      }
      else {
        $route = 'media_folder_management.file_explorer';
      }
      $link = Link::fromTextAndUrl('', Url::fromRoute($route))->toRenderable();
      $link['#attributes']['title'] = $this->t('Parent folder');
      $link['#attributes']['data-label'] = $this->t('Parent folder');

      return $link;
    }
    elseif (isset($parent->parent->target_id)) {
      $folder_route = $this->getFolderRoute($this->routeMatch->getRouteName());
      $link = Link::fromTextAndUrl('', Url::fromRoute($folder_route, ['folder' => $parent->parent->target_id]))->toRenderable();
      $link['#attributes']['title'] = $this->t('Parent folder');
      $link['#attributes']['data-label'] = $this->t('Parent folder');

      return $link;
    }

    return Markup::create('<span></span>');
  }

  /**
   * Get Breadcrumb.
   */
  public function getBreadcrumb($parent) {
    if (isset($parent->parent->target_id)) {
      $folder_route = $this->getFolderRoute($this->routeMatch->getRouteName());
      $parents = [];
      $term = clone $parent;
      $parents[$parent->id()] = '<span>' . Unicode::truncate($parent->getName(), 15, FALSE, TRUE) . '</span>';
      while (!empty($term->parent->target_id)) {
        $term = $term->parent->entity;
        $parents[$term->id()] = Link::fromTextAndUrl(Unicode::truncate($term->getName(), 15, FALSE, TRUE), Url::fromRoute($folder_route, ['folder' => $term->id()]))->toString();
      }
      $parents[0] = Link::fromTextAndUrl($this->t('Root'), Url::fromRoute('media_folder_management.file_explorer'))->toString();
      $parents = array_reverse($parents);

      return Markup::create(implode('<i></i>', $parents));
    }

    return Markup::create('<span class="root">' . $this->t('Root') . '</span>');
  }

  /**
   * Get folder route.
   */
  public function getFolderRoute($current_route_name) {
    if (str_starts_with($current_route_name, 'media_folder_management.collection')) {
      return 'media_folder_management.collection.folder';
    }

    return 'media_folder_management.folder';
  }

  /**
   * Get Buttons.
   */
  private function getButtons() {
    $action_links = [];

    $destination = $this->redirectDestination->getAsArray();
    if (!empty($destination['destination'])) {
      $url = parse_url($destination['destination']);
      $destination['destination'] = str_replace('/ajax', '', $url['path']);
    }

    $action_links['sorts'] = [
      'title' => $this->t('Sort by'),
      'url' => Url::fromUri('internal://<nolink>'),
      'attributes' => [
        'class' => ['sorts'],
      ],
    ];
    $action_links['az'] = [
      'title' => $this->t('Alphabetic ascending'),
      'url' => Url::fromRoute('media_folder_management.cookies', [
        'name' => 'view-order',
        'value' => 'az',
      ], ['query' => $destination]),
      'attributes' => [
        'class' => ['az'],
      ],
    ];
    $action_links['za'] = [
      'title' => $this->t('Alphabetic descending'),
      'url' => Url::fromRoute('media_folder_management.cookies', [
        'name' => 'view-order',
        'value' => 'za',
      ], ['query' => $destination]),
      'attributes' => [
        'class' => ['za'],
      ],
    ];
    $action_links['date-asc'] = [
      'title' => $this->t('Date ascending'),
      'url' => Url::fromRoute('media_folder_management.cookies', [
        'name' => 'view-order',
        'value' => 'date-asc',
      ], ['query' => $destination]),
      'attributes' => [
        'class' => ['date-asc'],
      ],
    ];
    $action_links['date-desc'] = [
      'title' => $this->t('Date descending'),
      'url' => Url::fromRoute('media_folder_management.cookies', [
        'name' => 'view-order',
        'value' => 'date-desc',
      ], ['query' => $destination]),
      'attributes' => [
        'class' => ['date-desc'],
      ],
    ];

    $order = $this->getCookie('view-order', $this->mediaFolderFileSettings->get('default_order'));
    $action_links[$order]['attributes']['class'][] = 'active';
    $action_links['sorts']['attributes']['class'][] = $order;

    $actions = [
      '#type' => 'dropbutton',
      '#links' => $action_links,
    ];

    $buttons = $render_buttons = [];
    $buttons['thumbs'] = $this->createLink($this->t('Grid'), Url::fromRoute('media_folder_management.cookies', [
      'name' => 'view-type',
      'value' => 'thumbs',
    ]), ['thumbs'], $this->t('Folder view'), TRUE);
    $buttons['list'] = $this->createLink($this->t('List'), Url::fromRoute('media_folder_management.cookies', [
      'name' => 'view-type',
      'value' => 'list',
    ]), ['list'], $this->t('List view'), TRUE);
    $view = $this->getCookie('view-type', $this->mediaFolderFileSettings->get('default_view'));
    $buttons[$view]['#attributes']['class'][] = 'active';

    foreach ($buttons as $button) {
      $render_buttons[] = $this->renderer->render($button);
    }

    return $this->renderer->render($actions) . implode('', $render_buttons);
  }

  /**
   * Get Item Actions.
   */
  private function getItemActions(mixed $folder = FALSE, mixed $file = FALSE) {
    $items = $render_items = [];

    if ($folder) {
      if (MediaFolderFileAccessCheck::hasEditFolderPermission($folder, $this->currentUser, TRUE)) {
        $items['edit'] = $this->createDialogLink($this->t('Edit folder'), Url::fromRoute('media_folder_management.edit_folder', ['folder' => $folder->id()]), ['edit-action'], $this->t('Edit'), TRUE, '90%');
      }

      if (MediaFolderFileAccessCheck::hasDeleteFolderPermission($folder, $this->currentUser, TRUE)) {
        $items['delete'] = $this->createDialogLink($this->t('Delete folder'), Url::fromRoute('media_folder_management.delete_folder', ['folder' => $folder->id()]), ['delete-action'], $this->t('Delete'), FALSE, '90%');
      }

      if (MediaFolderFileAccessCheck::hasAdminFolderPermission($this->currentUser)) {
        $items['perms'] = $this->createDialogLink($this->t('Folder permissions'), Url::fromRoute('media_folder_management.folder_perms', ['folder' => $folder->id()]), ['perms-action'], $this->t('Permissions'), TRUE, '90%');
      }

      if (MediaFolderFileAccessCheck::hasViewFolderPermission($folder, $this->currentUser, TRUE)) {
        $items['share'] = $this->createDialogLink($this->t('Share link'), Url::fromRoute('media_folder_management.share', ['folder' => $folder->id()]), ['share-action'], $this->t('Share link'), TRUE, '90%');
      }
    }
    elseif ($file) {
      if (MediaFolderFileAccessCheck::hasLockFileAccess($file, $this->currentUser, $this->entityTypeManager->getStorage('taxonomy_term'))) {
        $text = ($file->field_locked->value) ? $this->t('Unlock') : $this->t('Lock');
        $class = ($file->field_locked->value) ? ['unlock-action'] : ['lock-action'];
        $items['lock'] = $this->createDialogLink($text, Url::fromRoute('entity.media_folder_file.lock', ['media_folder_file' => $file->id()]), $class, $text, TRUE);
      }
      if (MediaFolderFileAccessCheck::hasEditFileAccess($file, $this->currentUser, $this->entityTypeManager->getStorage('taxonomy_term'))) {
        $items['edit'] = $this->createDialogLink($this->t('Edit file'), Url::fromRoute('entity.media_folder_file.edit_form', ['media_folder_file' => $file->id()]), ['edit-action'], $this->t('Edit'), TRUE);
      }
      if (MediaFolderFileAccessCheck::hasFileRevisionAccess($file, $this->currentUser, $this->entityTypeManager->getStorage('taxonomy_term'))) {
        $items['revision'] = $this->createDialogLink($this->t('File revisions'), Url::fromRoute('entity.media_folder_file.version_history', ['media_folder_file' => $file->id()]), ['revisions-action'], $this->t('RevisionsEdit'), TRUE, '70%');
      }
      if (MediaFolderFileAccessCheck::hasDeleteFileAccess($file, $this->currentUser, $this->entityTypeManager->getStorage('taxonomy_term'))) {
        $items['delete'] = $this->createDialogLink($this->t('Delete file'), Url::fromRoute('entity.media_folder_file.delete_form', ['media_folder_file' => $file->id()]), ['delete-action'], $this->t('Delete'), TRUE);
      }
    }

    foreach ($items as $item) {
      $render_items[] = $this->renderer->render($item);
    }

    return implode('', $render_items);
  }

  /**
   * Get human Filesize.
   */
  private function humanFilesize($bytes, $decimals = 2) {
    $size = ['B', 'kB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
    $factor = floor((strlen($bytes) - 1) / 3);

    return (!empty($size[$factor])) ? sprintf("%.{$decimals}f", $bytes / pow(1024, $factor)) . $size[$factor] : '';
  }

  /**
   * Get Locked Info.
   */
  private function getLockedInfo(mixed $file = FALSE) {
    $items = [];

    if ($file && $file->field_locked->value) {
      $items[] = ' data-locked-by="' . $file->getRevisionLogMessage() . '"';
    }

    return implode('', $items);
  }

  /**
   * Get Form.
   */
  private function getForm() {
    $form = $this->formBuilder->getForm('\Drupal\media_folder_management\Form\MediaFolderFileSearchForm');

    return $this->renderer->render($form);
  }

  /**
   * Builds an AJAX response for media details.
   */
  public function ajaxMediaDetail(MediaFolderFile $media_folder_file) {
    $view_builder = $this->entityTypeManager->getViewBuilder('media_folder_file');
    $preview = $view_builder->view($media_folder_file, 'default');

    return [
      'render' => $this->renderer->render($preview),
    ];
  }

}
